@@ -8,11 +8,8 @@ from .pyfiles import (  # noqa
     pyc_file_from_path, load_python_file, edit)
 from .sqla_compat import (  # noqa
     sqla_07, sqla_079, sqla_08, sqla_083, sqla_084, sqla_09, sqla_092,
-    sqla_094, sqla_094, sqla_099, sqla_100, sqla_105)
-
-
-class CommandError(Exception):
-    pass
+    sqla_094, sqla_099, sqla_100, sqla_105, sqla_110)
+from .exc import CommandError
 
 
 if not sqla_07:
