@@ -3,7 +3,8 @@ import sys
 from sqlalchemy import MetaData, Column, Table, Integer, String, Text, \
     Numeric, CHAR, ForeignKey, INTEGER, Index, UniqueConstraint, \
     TypeDecorator, CheckConstraint, text, PrimaryKeyConstraint, \
-    ForeignKeyConstraint
+    ForeignKeyConstraint, VARCHAR, DECIMAL, DateTime, BigInteger, BIGINT, \
+    SmallInteger
 from sqlalchemy.types import NULLTYPE
 from sqlalchemy.engine.reflection import Inspector
 
@@ -13,7 +14,7 @@ from alembic.migration import MigrationContext
 from alembic.testing import TestBase
 from alembic.testing import config
 from alembic.testing import assert_raises_message
-from alembic.testing.mock import Mock
+from alembic.testing.mock import Mock, patch
 from alembic.testing import eq_, is_, is_not_
 from alembic.util import CommandError
 from ._autogen_fixtures import AutogenTest, AutogenFixtureTest
@@ -187,6 +188,35 @@ class AutogenDefaultSchemaTest(AutogenFixtureTest, TestBase):
         eq_(diffs[0][1].c.keys(), ['x'])
 
 
+class AutogenDefaultSchemaIsNoneTest(AutogenFixtureTest, TestBase):
+    __only_on__ = 'sqlite'
+
+    def setUp(self):
+        super(AutogenDefaultSchemaIsNoneTest, self).setUp()
+
+        # prerequisite
+        eq_(self.bind.dialect.default_schema_name, None)
+
+    def test_no_default_schema(self):
+
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('a', m1, Column('x', String(50)))
+        Table('a', m2, Column('x', String(50)))
+
+        def _include_object(obj, name, type_, reflected, compare_to):
+            if type_ == "table":
+                return name in 'a' and obj.schema != 'main'
+            else:
+                return True
+
+        diffs = self._fixture(
+            m1, m2, include_schemas=True,
+            object_filters=_include_object)
+        eq_(len(diffs), 0)
+
+
 class ModelOne(object):
     __requires__ = ('unique_constraint_reflection', )
 
@@ -554,6 +584,63 @@ class AutogenerateDiffTestWSchema(ModelOne, AutogenTest, TestBase):
         eq_(diffs[10][3].name, 'pw')
 
 
+class CompareTypeSpecificityTest(TestBase):
+    def _fixture(self):
+        from alembic.ddl import impl
+        from sqlalchemy.engine import default
+
+        return impl.DefaultImpl(
+            default.DefaultDialect(), None, False, True, None, {})
+
+    def test_string(self):
+        t1 = String(30)
+        t2 = String(40)
+        t3 = VARCHAR(30)
+        t4 = Integer
+
+        impl = self._fixture()
+        is_(impl.compare_type(Column('x', t3), Column('x', t1)), False)
+        is_(impl.compare_type(Column('x', t3), Column('x', t2)), True)
+        is_(impl.compare_type(Column('x', t3), Column('x', t4)), True)
+
+    def test_numeric(self):
+        t1 = Numeric(10, 5)
+        t2 = Numeric(12, 5)
+        t3 = DECIMAL(10, 5)
+        t4 = DateTime
+
+        impl = self._fixture()
+        is_(impl.compare_type(Column('x', t3), Column('x', t1)), False)
+        is_(impl.compare_type(Column('x', t3), Column('x', t2)), True)
+        is_(impl.compare_type(Column('x', t3), Column('x', t4)), True)
+
+    def test_integer(self):
+        t1 = Integer()
+        t2 = SmallInteger()
+        t3 = BIGINT()
+        t4 = String()
+        t5 = INTEGER()
+        t6 = BigInteger()
+
+        impl = self._fixture()
+        is_(impl.compare_type(Column('x', t5), Column('x', t1)), False)
+        is_(impl.compare_type(Column('x', t3), Column('x', t1)), True)
+        is_(impl.compare_type(Column('x', t3), Column('x', t6)), False)
+        is_(impl.compare_type(Column('x', t3), Column('x', t2)), True)
+        is_(impl.compare_type(Column('x', t5), Column('x', t2)), True)
+        is_(impl.compare_type(Column('x', t1), Column('x', t4)), True)
+
+    def test_datetime(self):
+        t1 = DateTime()
+        t2 = DateTime(timezone=False)
+        t3 = DateTime(timezone=True)
+
+        impl = self._fixture()
+        is_(impl.compare_type(Column('x', t1), Column('x', t2)), False)
+        is_(impl.compare_type(Column('x', t1), Column('x', t3)), True)
+        is_(impl.compare_type(Column('x', t2), Column('x', t3)), True)
+
+
 class AutogenerateCustomCompareTypeTest(AutogenTest, TestBase):
     __only_on__ = 'sqlite'
 
